import glob
import os
import sqlite3
import artifacts.artGlobals

from packaging import version
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv, kmlgen, is_platform_windows, open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class RoutineDLocationsLocal(ab.Artifact):

    _name = 'RoutineD Locations Entry'
    _search_dirs = (
        '**/private/var/mobile/Library/Caches/com.apple.routined/Local.sqlite*'
    )
    _category = 'Locations'

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) < version.parse("12"):
            logfunc(
                "Unsupported version for RoutineD Locations Local.sqlite on iOS "
                + iOSversion
            )
        else:
            for file_found in files_found:
                file_found = str(file_found)

                if file_found.endswith('Local.sqlite'):
                    break

            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()
            if version.parse(iOSversion) >= version.parse("14"):  # Tested 14.1
                cursor.execute(
                    """
                select
                datetime(zrtlearnedlocationofinterestvisitmo.zentrydate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinterestvisitmo.zexitdate + 978307200, 'unixepoch'),
                (zrtlearnedlocationofinterestvisitmo.zexitdate-zrtlearnedlocationofinterestvisitmo.zentrydate)/60.00,
                zrtlearnedlocationofinterestmo.zlocationlatitude,
                zrtlearnedlocationofinterestmo.zlocationlongitude,
                zrtlearnedlocationofinterestvisitmo.zconfidence,
                zrtlearnedlocationofinterestvisitmo.zlocationverticaluncertainty,
                zrtlearnedlocationofinterestvisitmo.zlocationhorizontaluncertainty,
                zrtlearnedlocationofinterestvisitmo.zdatapointcount,
                datetime(zrtlearnedlocationofinterestvisitmo.zcreationdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinterestvisitmo.zexpirationdate + 978307200, 'unixepoch'),
                zrtlearnedlocationofinterestvisitmo.zlocationlatitude,
                zrtlearnedlocationofinterestvisitmo.zlocationlongitude
                from
                zrtlearnedlocationofinterestvisitmo,
                zrtlearnedlocationofinterestmo
                where zrtlearnedlocationofinterestmo.z_pk = zrtlearnedlocationofinterestvisitmo.zlocationofinterest
                """
                )

                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                data_list = []
                if usageentries > 0:
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                                row[8],
                                row[9],
                                row[10],
                                row[11],
                                row[12],
                            )
                        )

                    description = 'Significant Locations - Location of Interest Entry (Historical)'
                    report = ArtifactHtmlReport('Locations')
                    report.start_artifact_report(
                        report_folder, 'RoutineD Locations Entry', description
                    )
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'Exit',
                        'Entry Time (Minutes)',
                        'Latitude',
                        'Longitude',
                        'Confidence',
                        'Location Vertical Uncertainty',
                        'Location Horizontal Uncertainty',
                        'Data Point Count',
                        'Place Creation Date',
                        'Expiration',
                        'Visit latitude',
                        'Visit Longitude',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'RoutineD Locations Entry'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'RoutineD Locations Entry'
                    timeline(report_folder, tlactivity, data_list, data_headers)

                    kmlactivity = 'RoutineD Locations Entry'
                    kmlgen(report_folder, kmlactivity, data_list, data_headers)

                else:
                    logfunc('No RoutineD Significant Locations Entry data available')

            else:  # < ios 14
                cursor.execute(
                    """
                select
                datetime(zrtlearnedlocationofinterestvisitmo.zentrydate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinterestvisitmo.zexitdate + 978307200, 'unixepoch'),
                (zrtlearnedlocationofinterestvisitmo.zexitdate-zrtlearnedlocationofinterestvisitmo.zentrydate)/60.00,
                zrtlearnedlocationofinterestmo.zlocationlatitude,
                zrtlearnedlocationofinterestmo.zlocationlongitude,
                zrtlearnedlocationofinterestvisitmo.zconfidence,
                zrtlearnedlocationofinterestvisitmo.zlocationuncertainty,
                zrtlearnedlocationofinterestvisitmo.zdatapointcount,
                datetime(zrtlearnedlocationofinterestvisitmo.zcreationdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinterestvisitmo.zexpirationdate + 978307200, 'unixepoch'),
                zrtlearnedlocationofinterestvisitmo.zlocationlatitude,
                zrtlearnedlocationofinterestvisitmo.zlocationlongitude
                from
                zrtlearnedlocationofinterestvisitmo,
                zrtlearnedlocationofinterestmo
                where zrtlearnedlocationofinterestmo.z_pk = zrtlearnedlocationofinterestvisitmo.zlocationofinterest
                """
                )

                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                data_list = []
                if usageentries > 0:
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                                row[8],
                                row[9],
                                row[10],
                                row[11],
                            )
                        )

                    description = 'Significant Locations - Location of Interest Entry (Historical)'
                    report = ArtifactHtmlReport('Locations')
                    report.start_artifact_report(
                        report_folder, 'RoutineD Locations Entry', description
                    )
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'Exit',
                        'Entry Time (Minutes)',
                        'Latitude',
                        'Longitude',
                        'Confidence',
                        'Location Uncertainty',
                        'Data Point Count',
                        'Place Creation Date',
                        'Expiration',
                        'Visit latitude',
                        'Visit Longitude',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'RoutineD Locations Entry'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'RoutineD Locations Entry'
                    timeline(report_folder, tlactivity, data_list, data_headers)

                    kmlactivity = 'RoutineD Locations Entry'
                    kmlgen(report_folder, kmlactivity, data_list, data_headers)

                else:
                    logfunc('No RoutineD Significant Locations Entry data available')

            if version.parse(iOSversion) >= version.parse("12"):
                cursor.execute(
                    """
                select
                datetime(zrtlearnedlocationofinteresttransitionmo.zstartdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinteresttransitionmo.zstopdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinteresttransitionmo.zcreationdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinteresttransitionmo.zexpirationdate + 978307200, 'unixepoch'),
                zrtlearnedlocationofinterestmo.zlocationlatitude,
                zrtlearnedlocationofinterestmo.zlocationlongitude
                from
                zrtlearnedlocationofinteresttransitionmo
                left join
                zrtlearnedlocationofinterestmo
                on zrtlearnedlocationofinterestmo.z_pk = zrtlearnedlocationofinteresttransitionmo.zlocationofinterest
                """
                )

                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                data_list = []
                if usageentries > 0:
                    for row in all_rows:
                        data_list.append(
                            (row[0], row[1], row[2], row[3], row[4], row[5])
                        )

                    description = 'Significant Locations - Location of Interest Transition Start (Historical)'
                    report = ArtifactHtmlReport('Locations')
                    report.start_artifact_report(
                        report_folder, 'RoutineD Transtition Start', description
                    )
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'Stop',
                        'Creation Date',
                        'Expiration',
                        'Latitude',
                        'Longitude',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'RoutineD Transtition Start'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'RoutineD Transtition Start'
                    timeline(report_folder, tlactivity, data_list, data_headers)

                    tlactivity = 'RoutineD Transtition Start'
                    timeline(report_folder, tlactivity, data_list, data_headers)

                else:
                    logfunc(
                        'No RoutineD Significant Locations Transtition Start data available'
                    )

            if version.parse(iOSversion) >= version.parse("12"):
                cursor.execute(
                    """
                select
                datetime(zrtlearnedlocationofinteresttransitionmo.zstartdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinteresttransitionmo.zstopdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinteresttransitionmo.zcreationdate + 978307200, 'unixepoch'),
                datetime(zrtlearnedlocationofinteresttransitionmo.zexpirationdate + 978307200, 'unixepoch'),
                zrtlearnedlocationofinterestmo.zlocationlatitude,
                zrtlearnedlocationofinterestmo.zlocationlongitude
                from
                zrtlearnedlocationofinteresttransitionmo, zrtlearnedlocationofinterestmo
                where zrtlearnedlocationofinterestmo.z_pk = zrtlearnedlocationofinteresttransitionmo.zlocationofinterest
                """
                )

                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                data_list = []
                if usageentries > 0:
                    for row in all_rows:
                        data_list.append(
                            (row[0], row[1], row[2], row[3], row[4], row[5])
                        )

                    description = 'Significant Locations - Location of Interest Transition Stop (Historical)'
                    report = ArtifactHtmlReport('Locations')
                    report.start_artifact_report(
                        report_folder, 'RoutineD Transtition Stop', description
                    )
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'Stop',
                        'Creation Date',
                        'Expiration',
                        'Latitude',
                        'Longitude',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'RoutineD Transtition Stop'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'RoutineD Transtition Stop'
                    timeline(report_folder, tlactivity, data_list, data_headers)

                    tlactivity = 'RoutineD Transtition Stop'
                    timeline(report_folder, tlactivity, data_list, data_headers)

                else:
                    logfunc(
                        'No RoutineD Significant Locations Transtition Stop data available'
                    )



import base64
import os
import re
import xml.etree.ElementTree as ET

from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv

from artifacts.Artifact import Artifact
from xleapp.html_report import Icon


class DeviceActivator(ab.Artifact):

    _name = 'iOS Device Activator Data'
    _search_dirs = (
        '*private/var/mobile/Library/Logs/mobileactivationd/ucrt_oob_request.txt'
    )
    _category = 'IOS Build'
    _web_icon = WebIcon.GIT_COMMIT

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        alllines = ''
        file_found = str(files_found[0])

        with open(file_found, 'r') as f_in:
            for line in f_in:
                line = line.strip()
                alllines = alllines + line

        found = re.findall(
            '<key>ActivationInfoXML</key><data>(.*)</data><key>RKCertification</key><data>',
            alllines,
        )
        base64_message = found[0]

        data = base64.b64decode(base64_message)

        outpath = os.path.join(report_folder, "results.xml")
        with open(outpath, 'wb') as f_out:
            f_out.write(data)

        xmlfile = outpath
        tree = ET.parse(xmlfile)
        root = tree.getroot()

        for elem in root:
            for elemx in elem:
                for elemz in elemx:
                    data_list.append(str(elemz.text).strip())

        it = iter(data_list)
        results = list(zip(it, it))

        for x in results:
            if x[0] == 'EthernetMacAddress':
                logdevinfo(f"Ethernet Mac Address: {x[1]}")
            if x[0] == 'BluetoothAddress':
                logdevinfo(f"Bluetooth Address: {x[1]}")
            if x[0] == 'WifiAddress':
                logdevinfo(f"Wifi Address: {x[1]}")
            if x[0] == 'ModelNumber':
                logdevinfo(f"Model Number: {x[1]}")

        if len(results) > 0:
            report = ArtifactHtmlReport('iOS Device Activator Data')
            report.start_artifact_report(report_folder, 'iOS Device Activator Data')
            report.add_script()
            data_headers = ('Key', 'Values')
            report.write_artifact_data_table(data_headers, results, file_found)
            report.end_artifact_report()

            tsvname = 'iOS Device Activator Data'
            tsv(report_folder, data_headers, results, tsvname)
        else:
            logfunc('No iOS Device Activator Data')



import datetime
import os
import re
import sqlite3
import sys
import time

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, open_sqlite_db_readonly, tsv

from artifacts.Artifact import Artifact


class MediaLibrary(ab.Artifact):

    _name = 'Media Library'
    _search_dirs = '**/Medialibrary.sqlitedb'
    _category = 'Media Library'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        # Execute query for retrieving media information
        try:
            cursor.execute(
                """
            SELECT ext.title, ext.media_kind, itep.format,
                    ext.location, ext.total_time_ms, ext.file_size, ext.year,
                    alb.album, alba.album_artist, com.composer, gen.genre,
                    ite.track_number, art.artwork_token,
                    itev.extended_content_rating, itev.movie_info,
                    ext.description_long, sto.account_id,
                    strftime(\'%d/%m/%Y %H:%M:%S\',
                    datetime(sto.date_purchased + 978397200, \'unixepoch\'))
                    date_purchased, sto.store_item_id, sto.purchase_history_id, ext.copyright
                    from item_extra
                    ext join item_store sto using (item_pid)
                    join item ite using (item_pid)
                    join item_stats ites using (item_pid)
                    join item_playback itep using (item_pid)
                    join item_video itev using (item_pid)
                    left join album alb on sto.item_pid=alb.representative_item_pid
                    left join album_artist alba
                    on sto.item_pid=alba.representative_item_pid
                    left join composer com on
                    sto.item_pid=com.representative_item_pid
                    left join genre gen on sto.item_pid=gen.representative_item_pid
                    left join item_artist itea on
                    sto.item_pid=itea.representative_item_pid
                    left join artwork_token art on sto.item_pid=art.entity_pid
            """
            )
        except:
            logfunc('Error executing SQLite query')

        all_rows = cursor.fetchall()
        data_list = []
        media_type = ''

        for row in all_rows:
            col_count = 0
            tmp_row = []

            for media_type in row:
                if col_count == 1:
                    if media_type == 0:
                        media_type = "E-book"
                    if media_type == 1:
                        media_type = "Audio"
                    if media_type == 2:
                        media_type = "Film"
                    if media_type == 33:
                        media_type = "Video M4V"
            if media_type == 4:
                media_type = "Podcast"
                if col_count == 4:
                    media_type = int(media_type)

                col_count = col_count + 1
                tmp_row.append(str(media_type).replace('\n', ''))

            data_list.append(tmp_row)

        # Recover account information
        data_list_info = []
        cursor.execute(
            """
        select * from _MLDATABASEPROPERTIES;
        """
        )
        iOS_info = cursor.fetchall()

        iCloud_items = [
            'MLJaliscoAccountID',
            'MPDateLastSynced',
            'MPDateToSyncWithUbiquitousStore',
            'OrderingLanguage',
        ]

        for row in iOS_info:
            for elm in iCloud_items:
                if row[0] == elm:
                    data_list_info.append((row[0], row[1]))

        report = ArtifactHtmlReport('Media Library')
        report.start_artifact_report(report_folder, 'Media Library')
        report.add_script()
        data_headers_info = ('key', 'value')
        data_headers = (
            'Title',
            'Media Type',
            'File Format',
            'File',
            'Total Time (ms)',
            'File Size',
            'Year',
            'Album Name',
            'Album Artist',
            'Composer',
            'Genre',
            'Track Number',
            'Artwork',
            'Content Rating',
            'Movie Information',
            'Description',
            'Account ID',
            'Date Purchased',
            'Item ID',
            'Purchase History ID',
            'Copyright',
        )

        report.write_artifact_data_table(
            data_headers_info, data_list_info, file_found, cols_repeated_at_bottom=False
        )
        report.write_artifact_data_table(
            data_headers, data_list, file_found, True, False
        )

        report.end_artifact_report()

        tsvname = 'Media Library'
        tsv(report_folder, data_headers_info, data_list_info, tsvname)
        tsv(report_folder, data_headers, data_list, tsvname)



import os
import plistlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, logfunc

from artifacts.Artifact import Artifact


class IconsScreen(ab.Artifact):

    _name = 'Apps per screen'
    _search_dirs = '**/SpringBoard/IconState.plist'
    _category = 'iOS Screens'

    def get(self, files_found, seeker):
        data_list = []
        data_pre_list = []
        file_found = str(files_found[0])
        with open(file_found, "rb") as fp:
            plist = plistlib.load(fp)
            for key, val in plist.items():
                if key == "buttonBar":
                    bbar = val
                elif key == "iconLists":
                    icon = val

            for x in range(0, len(icon)):
                page = icon[x]
                htmlstring = f"<table><tr>"
                htmlstring = htmlstring + (f'<td colspan="4"> Icons screen #{x}</td>')
                for y in range(0, len(page)):
                    rows = page[y]
                    if (y == 0) or (y % 4 == 0):
                        htmlstring = htmlstring + ("</tr><tr>")

                    if isinstance(rows, dict):
                        var = rows
                        foldername = var['displayName']
                        rows = f'Folder: {foldername}'
                        bundlesinfolder = var['iconLists'][0]
                        for items in bundlesinfolder:
                            rows = rows + '<br>' + items

                    htmlstring = htmlstring + (f"<td width = 25%>{rows}</td>")
                htmlstring = htmlstring + ("</tr></table>")
                data_list.append((htmlstring,))

            htmlstring = ''
            htmlstring = f'<table><tr> <td colspan="4"> Icons bottom bar</td></tr><tr>'
            for x in range(0, len(bbar)):
                htmlstring = htmlstring + (f"<td width = 25%>{bbar[x]}</td>")
            htmlstring = htmlstring + ("</tr></table>")
            data_list.append((htmlstring,))

            logfunc("Screens: " + str(len(icon)))

            report = ArtifactHtmlReport(f'Apps per screen')
            report.start_artifact_report(report_folder, f'Apps per screen')
            report.add_script()
            data_headers = (f'Apps per Screens',)
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()



import os
import shutil

import pandas as pd
from html_report.artifact_report import ArtifactHtmlReport
from html_report.chat_rendering import chat_HTML, render_chat
from helpers import (
    is_platform_windows,
    open_sqlite_db_readonly,
    sanitize_file_name,
    timeline,
    tsv,
)

from artifacts.Artifact import Artifact


class SMS(ab.Artifact):
    _name = 'SMS'
    _search_dirs = '**/sms.db'
    _category = 'SMS & iMessage'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        sms_df = pd.read_sql_query(
            """
        SELECT
        CASE
            WHEN LENGTH(MESSAGE.DATE)=18 THEN DATETIME(MESSAGE.DATE/1000000000+978307200,'UNIXEPOCH')
            WHEN LENGTH(MESSAGE.DATE)=9 THEN DATETIME(MESSAGE.DATE + 978307200,'UNIXEPOCH')
            ELSE "N/A"
        END "MESSAGE DATE",
        MESSAGE.ROWID as "MESSAGE ID",
        CASE
            WHEN LENGTH(MESSAGE.DATE_DELIVERED)=18 THEN DATETIME(MESSAGE.DATE_DELIVERED/1000000000+978307200,"UNIXEPOCH")
            WHEN LENGTH(MESSAGE.DATE_DELIVERED)=9 THEN DATETIME(MESSAGE.DATE_DELIVERED+978307200,"UNIXEPOCH")
            ELSE "N/A"
        END "DATE DELIVERED",
        CASE
            WHEN LENGTH(MESSAGE.DATE_READ)=18 THEN DATETIME(MESSAGE.DATE_READ/1000000000+978307200,"UNIXEPOCH")
            WHEN LENGTH(MESSAGE.DATE_READ)=9 THEN DATETIME(MESSAGE.DATE_READ+978307200,"UNIXEPOCH")
            ELSE "N/A"
        END "DATE READ",
        MESSAGE.TEXT as "MESSAGE",
        HANDLE.ID AS "CONTACT ID",
        MESSAGE.SERVICE AS "SERVICE",
        MESSAGE.ACCOUNT AS "ACCOUNT",
        MESSAGE.IS_DELIVERED AS "IS DELIVERED",
        MESSAGE.IS_FROM_ME AS "IS FROM ME",
        ATTACHMENT.FILENAME AS "FILENAME",
        ATTACHMENT.MIME_TYPE AS "MIME TYPE",
        ATTACHMENT.TRANSFER_NAME AS "TRANSFER TYPE",
        ATTACHMENT.TOTAL_BYTES AS "TOTAL BYTES"
        FROM MESSAGE
        LEFT OUTER JOIN MESSAGE_ATTACHMENT_JOIN ON MESSAGE.ROWID = MESSAGE_ATTACHMENT_JOIN.MESSAGE_ID
        LEFT OUTER JOIN ATTACHMENT ON MESSAGE_ATTACHMENT_JOIN.ATTACHMENT_ID = ATTACHMENT.ROWID
        LEFT OUTER JOIN HANDLE ON MESSAGE.HANDLE_ID = HANDLE.ROWID
        """,
            db,
        )

        usageentries = sms_df.shape[0]
        if usageentries > 0:
            data_list = sms_df.to_records(index=False)

            report = ArtifactHtmlReport('SMS - iMessage')
            report.start_artifact_report(report_folder, 'SMS - iMessage')
            report.add_script()

            sms_df = sms_df.rename(
                columns={
                    "MESSAGE DATE": "data-time",
                    'MESSAGE ID': "message-id",
                    "MESSAGE": "message",
                    "CONTACT ID": "data-name",
                    "IS FROM ME": "from_me",
                    "MIME TYPE": "content-type",
                }
            )
            sms_df["data-time"] = pd.to_datetime(sms_df["data-time"])

            def copyAttachments(rec):
                pathToAttachment = None
                if rec["FILENAME"]:
                    attachment = seeker.search(
                        '**' + rec["FILENAME"].replace('~', '', 1),
                        return_on_first_hit=True,
                    )
                    if not attachment:
                        logfunc(
                            ' [!] Unable to extract attachment file: "{}"'.format(
                                rec['FILENAME']
                            )
                        )
                        return
                    if is_platform_windows():
                        destFileName = sanitize_file_name(
                            os.path.basename(rec["FILENAME"])
                        )
                    else:
                        destFileName = os.path.basename(rec["FILENAME"])
                    pathToAttachment = os.path.join(
                        (os.path.basename(os.path.abspath(report_folder))), destFileName
                    )
                    shutil.copy(
                        attachment[0], os.path.join(report_folder, destFileName)
                    )
                return pathToAttachment

            sms_df["file-path"] = sms_df.apply(lambda rec: copyAttachments(rec), axis=1)

            num_entries = sms_df.shape[0]
            report.write_minor_header(f'Total number of entries: {num_entries}', 'h6')

            if file_found.startswith('\\\\?\\'):
                file_found = file_found[4:]
            report.write_lead_text(f'SMS - iMessage located at: {file_found}')
            report.write_raw_html(chat_HTML)
            report.add_script(render_chat(sms_df))

            data_headers = (
                'Message Date',
                'Message ID',
                'Date Delivered',
                'Date Read',
                'Message',
                'Contact ID',
                'Service',
                'Account',
                'Is Delivered',
                'Is from Me',
                'Filename',
                'MIME Type',
                'Transfer Type',
                'Total Bytes',
            )
            report.write_artifact_data_table(
                data_headers,
                data_list,
                file_found,
                write_total=False,
                write_location=False,
            )
            report.end_artifact_report()

            tsvname = 'SMS - iMessage'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'SMS - iMessage'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No SMS & iMessage data available')

        db.close()



import io
import os

import nska_deserialize as nd
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv
from helpers.db import open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class CloudKitNotSharing(ab.Artifact):

    _name = 'CloudKit Note Sharing'
    _search_dirs = '*NoteStore.sqlite*'
    _category = 'CloudKit'
    _web_icon = WebIcon.SHARE_2

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('NoteStore.sqlite'):
                db = open_sqlite_db_readonly(file_found)
                cursor = db.cursor()
                cursor.execute(
                    """
                select z_pk, zserverrecorddata
                from
                ziccloudsyncingobject
                where
                zserverrecorddata not null
                """
                )

                note_data = []
                all_rows = cursor.fetchall()
                result_number = len(all_rows)
                if result_number > 0:

                    for row in all_rows:

                        filename = os.path.join(
                            report_folder,
                            'zserverrecorddata_' + str(row[0]) + '.bplist',
                        )
                        output_file = open(filename, "wb")
                        output_file.write(row[1])
                        output_file.close()

                        deserialized_plist = nd.deserialize_plist(io.BytesIO(row[1]))
                        creator_id = ''
                        last_modified_id = ''
                        creation_date = ''
                        last_modified_date = ''
                        last_modified_device = ''
                        record_type = ''
                        record_id = ''
                        for item in deserialized_plist:
                            if 'RecordCtime' in item:
                                creation_date = item['RecordCtime']
                            elif 'RecordMtime' in item:
                                last_modified_date = item['RecordMtime']
                            elif 'LastModifiedUserRecordID' in item:
                                last_modified_id = item['LastModifiedUserRecordID'][
                                    'RecordName'
                                ]
                            elif 'CreatorUserRecordID' in item:
                                creator_id = item['CreatorUserRecordID']['RecordName']
                            elif 'ModifiedByDevice' in item:
                                last_modified_device = item['ModifiedByDevice']
                            elif 'RecordType' in item:
                                record_type = item['RecordType']
                            elif 'RecordID' in item:
                                record_id = item['RecordID']['RecordName']

                        note_data.append(
                            [
                                record_id,
                                record_type,
                                creation_date,
                                creator_id,
                                last_modified_date,
                                last_modified_id,
                                last_modified_device,
                            ]
                        )

                    description = (
                        'CloudKit Note Sharing - Notes information '
                        'shared via CloudKit. Look up the '
                        'Record ID in the ZICCLOUDSYYNCINGOBJECT.'
                        'ZIDENTIFIER column.'
                    )
                    report = ArtifactHtmlReport('Note Sharing')
                    report.start_artifact_report(
                        report_folder, 'Note Sharing', description
                    )
                    report.add_script()
                    note_headers = (
                        'Record ID',
                        'Record Type',
                        'Creation Date',
                        'Creator ID',
                        'Modified Date',
                        'Modifier ID',
                        'Modifier Device',
                    )
                    report.write_artifact_data_table(
                        note_headers, note_data, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'CloudKit Note Sharing'
                    tsv(report_folder, note_headers, note_data, tsvname)
                else:
                    logfunc('No CloudKit - CloudKit Note Sharing data available')

                db.close()



import os
import plistlib
from datetime import datetime

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv

from artifacts.Artifact import Artifact


class ICloudWifi(ab.Artifact):

    _name = 'iCloud Wifi Networks'
    _search_dirs = '**/com.apple.wifid.plist'
    _category = 'Wifi Connections'

    def get(self, files_found, seeker):
        data_list = []
        file_found = str(files_found[0])
        with open(file_found, 'rb') as fp:
            pl = plistlib.load(fp)
            timestamp = ''

            if 'values' in pl.keys():
                for key, val in pl['values'].items():
                    network_name = key

                    if type(val) == dict:
                        for key2, val2 in val.items():
                            if key2 == 'value':
                                if type(val2) == dict:
                                    if 'BSSID' in val2:
                                        bssid = str(val2['BSSID'])
                                    else:
                                        bssid = 'Not Available'

                                    if 'SSID_STR' in val2:
                                        ssid = str(val2['SSID_STR'])
                                    else:
                                        ssid = 'Not Available'

                                    if 'added_by' in val2:
                                        added_by = str(val2['added_by'])
                                    else:
                                        added_by = 'Not Available'

                                    if 'enabled' in val2:
                                        enabled = str(val2['enabled'])
                                    else:
                                        enabled = 'Not Available'

                                    if 'added_at' in val2:
                                        # Convert the value into a datetime object.
                                        my_time2 = str(val2['added_at'])
                                        datetime_obj = datetime.strptime(
                                            my_time2, '%b  %d %Y %H:%M:%S'
                                        )
                                        added_at = str(datetime_obj)
                                    else:
                                        added_at = 'Not Available'
                                    data_list.append(
                                        (bssid, ssid, added_by, enabled, added_at)
                                    )

        if len(data_list) > 0:
            report = ArtifactHtmlReport('iCloud Wifi Networks')
            report.start_artifact_report(report_folder, 'iCloud Wifi Networks')
            report.add_script()
            data_headers = ('BSSID', 'SSID', 'Added By', 'Enabled', 'Added At')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'iCloud Wifi Networks'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data on iCloud WiFi networks')



import glob
import json
import os
import pathlib
import plistlib
import sqlite3
import stat
import sys

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class LocationDSteps(ab.Artifact):
    _name = 'LocationD Steps'
    _search_dirs = '**/cache_encryptedC.db'
    _category = 'Locations'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("10"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(starttime + 978307200, 'unixepoch') as "start time",
            timestamp,
            count,
            distance,
            rawdistance,
            floorsascended,
            floorsdescended,
            pace,
            activetime,
            firststeptime,
            pushcount,
            workouttype
            from stepcounthistory
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD Steps')
                report.start_artifact_report(report_folder, 'Steps', description)
                report.add_script()
                data_headers = (
                    'Start Time',
                    'Movement Time',
                    'Count',
                    'Distance',
                    'Raw Distance',
                    'Floors Ascended',
                    'Floors Descended',
                    'Pace',
                    'Active Time',
                    'First Step Time',
                    'Push Count',
                    'Workout Type',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD Steps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'LocationD Steps'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available for Steps')

        elif version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(starttime + 978307200, 'unixepoch'),
            timestamp,
            count,
            distance,
            rawdistance,
            floorsascended,
            floorsdescended,
            pace,
            activetime
            from stepcounthistory
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD Steps')
                report.start_artifact_report(report_folder, 'Steps', description)
                report.add_script()
                data_headers = (
                    'Start Time',
                    'Movement Time',
                    'Count',
                    'Distance',
                    'Raw Distance',
                    'Floors Ascended',
                    'Floors Descended',
                    'Pace',
                    'Active Time',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD Steps'
                tsv(report_folder, data_headers, data_list, tsvname)
            else:
                logfunc('No data available for Steps')

        elif version.parse(iOSversion) >= version.parse("8"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(starttime + 978307200, 'unixepoch'),
            timestamp,
            count,
            distance,
            rawdistance,
            floorsascended,
            floorsdescended
            from stepcounthistory
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD Steps')
                report.start_artifact_report(report_folder, 'Steps', description)
                report.add_script()
                data_headers = (
                    'Start Time',
                    'Movement Time',
                    'Count',
                    'Distance',
                    'Raw Distance',
                    'Floors Ascended',
                    'Floors Descended',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD Steps'
                tsv(report_folder, data_headers, data_list, tsvname)
            else:
                logfunc('No data available for Steps')

        else:
            logfunc('No data available for Steps')

        db.close()



import glob
import json
import os
import pathlib
import plistlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class SafariTabs(ab.Artifact):
    _name = 'Safari Tabs'
    _search_dirs = '**/Safari/BrowserState.db'
    _category = 'Safari Browser'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()

        cursor.execute(
            """
        select
        datetime(last_viewed_time+978307200,'unixepoch'),
        title,
        url,
        user_visible_url,
        opened_from_link,
        private_browsing
        from
        tabs
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

            description = ''
            report = ArtifactHtmlReport('Safari Browser Tabs')
            report.start_artifact_report(report_folder, 'Tabs', description)
            report.add_script()
            data_headers = (
                'Last Viewed Time',
                'Title',
                'URL',
                'User Visible URL',
                'Opened from Link',
                'Private Browsing',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Safari Browser Tabs'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Safari Browser Tabs'
            timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No data available in table')

        db.close()



from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv
from helpers.db import open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class CoreDuetAireplane(ab.Artifact):

    _name = 'CoreDuet Airplane Mode'
    _search_dirs = '**/coreduetd.db'
    _category = 'CoreDuet'
    _web_icon = WebIcon.PAUSE

    def __init__(self):
        super().__init__(self)

    def get(files_found, report_folder, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()

        cursor.execute(
            """
        select
        datetime(zcreationdate+978307200,'unixepoch'),
        time(zlocaltime,'unixepoch'),
        time(zcreationdate-zlocaltime,'unixepoch'),
        case zairplanemodeon
        when "0" then "off"
        when "1" then "on"
        end
        from zcddmairplanemodeevent
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3]))

            description = ''
            report = ArtifactHtmlReport('CoreDuet Airplane Mode')
            report.start_artifact_report(report_folder, 'Airplane Mode', description)
            report.add_script()
            data_headers = (
                'Create Time',
                'Local Device Time',
                'Time Zone',
                'Airplane Mode',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Coreduet Airplane Mode'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Coreduet Airplane Mode'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in table')



from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv
from helpers.db import open_sqlite_db_readonly
from xleapp.html_report import Icon
from artifacts.Artifact import Artifact


class CoreDuetPlugin(ab.Artifact):

    _name = 'CoreDuet Plugged In'
    _search_dirs = '**/coreduetd.db'
    _category = 'CoreDuet'
    _web_icon = WebIcon.BATTERY_CHARGING

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()

        cursor.execute(
            """
            select
            datetime(zcreationdate+978307200,'unixepoch'),
            time(zcreationdate-zlocaltime,'unixepoch'),
            case zcablestate
            when '0' then 'unplugged'
            when '1' then 'plugged in'
            end
            from zcddmpluginevent
            """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3]))

            description = ''
            report = ArtifactHtmlReport('CoreDuet Plugged In')
            report.start_artifact_report(report_folder, 'Plugged In', description)
            report.add_script()
            data_headers = ('Timestamp', 'Time Zone', 'Cable State')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'CoreDuet Plugged In'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Coreduet Plugged In'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in table')



import datetime
import os
import re
import sqlite3
import string
import sys
import textwrap
from html import escape

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, timeline, tsv

from artifacts.Artifact import Artifact


class MobileInstall(ab.Artifact):
    _name = 'Mobile Installation Logs'
    _search_dirs = '**/mobile_installation.log.*'
    _category = 'Mobile Installation Logs'

    def get(self, files_found, seeker):
        counter = 0
        filescounter = 0
        tsv_tml_data_list = []

        mibdatabase = os.path.join(report_folder, 'mib.db')
        db = sqlite3.connect(mibdatabase)
        cursor = db.cursor()
        cursor.execute(
            """
        CREATE TABLE dimm(time_stamp TEXT, action TEXT, bundle_id TEXT, path TEXT)
        """
        )

        db.commit()

        for filename in files_found:
            file = open(filename, "r", encoding="utf8")
            filescounter = filescounter + 1
            for line in file:
                counter = counter + 1
                matchObj = re.search(
                    r"(Install Successful for)", line
                )  # Regex for installed applications
                if matchObj:
                    actiondesc = "Install successful"
                    matchObj1 = re.search(
                        r"(?<= for \(Placeholder:)(.*)(?=\))", line
                    )  # Regex for bundle id
                    matchObj2 = re.search(
                        r"(?<= for \(Customer:)(.*)(?=\))", line
                    )  # Regex for bundle id
                    matchObj3 = re.search(
                        r"(?<= for \(System:)(.*)(?=\))", line
                    )  # Regex for bundle id
                    matchObj4 = re.search(
                        r"(?<= for \()(.*)(?=\))", line
                    )  # Regex for bundle id
                    if matchObj1:
                        bundleid = matchObj1.group(1)
                    elif matchObj2:
                        bundleid = matchObj2.group(1)
                    elif matchObj3:
                        bundleid = matchObj3.group(1)
                    elif matchObj4:
                        bundleid = matchObj4.group(1)

                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    # logfunc(inserttime, actiondesc, bundleid)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        bundleid,
                        "",
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()
                    path = ''
                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))

                    # logfunc()

                matchObj = re.search(
                    r"(Destroying container with identifier)", line
                )  # Regex for destroyed containers
                if matchObj:
                    actiondesc = "Destroying container"
                    # logfunc(actiondesc)
                    # logfunc("Destroyed containers:")
                    matchObj = re.search(
                        r"(?<=identifier )(.*)(?= at )", line
                    )  # Regex for bundle id
                    if matchObj:
                        bundleid = matchObj.group(1)
                        # logfunc ("Bundle ID: ", bundleid )

                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    matchObj = re.search(r"(?<= at )(.*)(?=$)", line)  # Regex for path
                    if matchObj:
                        path = matchObj.group(1)
                        # logfunc ("Path: ", matchObj.group(1))

                    # logfunc(inserttime, actiondesc, bundleid, path)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        bundleid,
                        path,
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()

                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))
                    # logfunc()

                matchObj = re.search(
                    r"(Data container for)", line
                )  # Regex Moved data containers
                if matchObj:
                    actiondesc = "Data container moved"
                    # logfunc(actiondesc)
                    # logfunc("Data container moved:")
                    matchObj = re.search(
                        r"(?<=for )(.*)(?= is now )", line
                    )  # Regex for bundle id
                    if matchObj:
                        bundleid = matchObj.group(1)
                        # logfunc ("Bundle ID: ", bundleid )

                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    matchObj = re.search(r"(?<= at )(.*)(?=$)", line)  # Regex for path
                    if matchObj:
                        path = matchObj.group(1)
                        # logfunc ("Path: ", matchObj.group(1))

                    # logfunc(inserttime, actiondesc, bundleid, path)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        bundleid,
                        path,
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()

                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))
                    # logfunc()

                matchObj = re.search(
                    r"(Made container live for)", line
                )  # Regex for made container
                if matchObj:
                    actiondesc = "Made container live"
                    # logfunc(actiondesc)
                    # logfunc("Made container:")
                    matchObj = re.search(
                        r"(?<=for )(.*)(?= at)", line
                    )  # Regex for bundle id
                    if matchObj:
                        bundleid = matchObj.group(1)
                        # logfunc ("Bundle ID: ", bundleid )

                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    matchObj = re.search(r"(?<= at )(.*)(?=$)", line)  # Regex for path
                    if matchObj:
                        path = matchObj.group(1)
                        # logfunc ("Path: ", matchObj.group(1))
                    # logfunc(inserttime, actiondesc, bundleid, path)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        bundleid,
                        path,
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()

                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))

                matchObj = re.search(
                    r"(Uninstalling identifier )", line
                )  # Regex for made container
                if matchObj:
                    actiondesc = "Uninstalling identifier"
                    # logfunc(actiondesc)
                    # logfunc("Uninstalling identifier")
                    matchObj = re.search(
                        r"(?<=Uninstalling identifier )(.*)", line
                    )  # Regex for bundle id
                    if matchObj:
                        bundleid = matchObj.group(1)
                        # logfunc ("Bundle ID: ", bundleid )

                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        bundleid,
                        "",
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()

                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))

                matchObj = re.search(
                    r"(main: Reboot detected)", line
                )  # Regex for reboots
                if matchObj:
                    actiondesc = "Reboot detected"
                    # logfunc(actiondesc)
                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        "",
                        "",
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()

                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))

                matchObj = re.search(
                    r"(Attempting Delta patch update of )", line
                )  # Regex for Delta patch
                if matchObj:
                    actiondesc = "Attempting Delta patch"
                    # logfunc(actiondesc)
                    # logfunc("Made container:")
                    matchObj = re.search(
                        r"(?<=Attempting Delta patch update of )(.*)(?= from)", line
                    )  # Regex for bundle id
                    if matchObj:
                        bundleid = matchObj.group(1)
                        # logfunc ("Bundle ID: ", bundleid )

                    matchObj = re.search(
                        r"(?<=^)(.*)(?= \[)", line
                    )  # Regex for timestamp
                    if matchObj:
                        timestamp = matchObj.group(1)
                        weekday, month, day, time, year = str.split(timestamp)
                        day = day_converter(day)
                        month = month_converter(month)
                        inserttime = (
                            str(year)
                            + "-"
                            + str(month)
                            + "-"
                            + str(day)
                            + " "
                            + str(time)
                        )
                        # logfunc(inserttime)
                        # logfunc(month)
                        # logfunc(day)
                        # logfunc(year)
                        # logfunc(time)
                        # logfunc ("Timestamp: ", timestamp)

                    matchObj = re.search(r"(?<= from )(.*)", line)  # Regex for path
                    if matchObj:
                        path = matchObj.group(1)
                        # logfunc ("Path: ", matchObj.group(1))
                    # logfunc(inserttime, actiondesc, bundleid, path)

                    # insert to database
                    cursor = db.cursor()
                    datainsert = (
                        inserttime,
                        actiondesc,
                        bundleid,
                        path,
                    )
                    cursor.execute(
                        "INSERT INTO dimm (time_stamp, action, bundle_id, path)  VALUES(?,?,?,?)",
                        datainsert,
                    )
                    db.commit()

                    tsv_tml_data_list.append((inserttime, actiondesc, bundleid, path))
                    # logfunc()

        logfunc(f"Logs processed: {filescounter}")
        logfunc(f"Lines processed: {counter}")
        logfunc("")
        file.close

        # Initialize counters
        totalapps = 0
        installedcount = 0
        uninstallcount = 0
        historicalcount = 0
        sysstatecount = 0

        # created folders for reports for App history
        os.makedirs(os.path.join(report_folder, "Apps_Historical"))

        data_list_installed = []
        data_list_uninstalled = []

        # Initialize database connection
        db = sqlite3.connect(mibdatabase)
        cursor = db.cursor()
        # Query to create installed and uninstalled app reports
        cursor.execute("""SELECT distinct bundle_id from dimm""")
        all_rows = cursor.fetchall()
        for row in all_rows:
            # logfunc(row[0])
            distinctbundle = row[0]
            cursor.execute(
                """SELECT * from dimm where bundle_id=? order by time_stamp desc limit 1""",
                (distinctbundle,),
            )
            all_rows_iu = cursor.fetchall()
            for row in all_rows_iu:
                # logfunc(row[0], row[1], row[2], row[3])
                if row[2] == "":
                    continue
                elif row[1] == "Destroying container":
                    # logfunc(row[0], row[1], row[2], row[3])
                    uninstallcount = uninstallcount + 1
                    totalapps = totalapps + 1
                    # tofile1 = row[0] + ' ' + row[1] + ' ' + row[2] + ' ' + row[3] + '\n'
                    data_list_uninstalled.append(
                        (
                            row[0],
                            row[2],
                        )
                    )
                    # logfunc()
                elif row[1] == "Uninstalling identifier":
                    # logfunc(row[0], row[1], row[2], row[3])
                    uninstallcount = uninstallcount + 1
                    totalapps = totalapps + 1
                    # tofile1 = row[0] + ' ' + row[1] + ' ' + row[2] + ' ' + row[3] + '\n'
                    data_list_uninstalled.append(
                        (
                            row[0],
                            row[2],
                        )
                    )
                    # logfunc()
                else:
                    # logfunc(row[0], row[1], row[2], row[3])
                    data_list_installed.append(
                        (
                            row[0],
                            row[2],
                        )
                    )
                    installedcount = installedcount + 1
                    totalapps = totalapps + 1

        location = f'{filename}'
        description = 'List of Uninstalled apps.'
        report = ArtifactHtmlReport('Apps - Uninstalled')
        report.start_artifact_report(report_folder, 'Apps - Uninstalled', description)
        report.add_script()
        data_headers = (
            'Last Uninstalled',
            'Bundle ID',
        )
        report.write_artifact_data_table(data_headers, data_list_uninstalled, location)
        report.end_artifact_report()

        location = f'{filename}'
        description = 'List of Installed apps.'
        report = ArtifactHtmlReport('Apps - Installed')
        report.start_artifact_report(report_folder, 'Apps - Installed', description)
        report.add_script()
        data_headers = (
            'Last Installed',
            'Bundle ID',
        )
        report.write_artifact_data_table(data_headers, data_list_installed, location)
        report.end_artifact_report()

        # Query to create historical report per app

        cursor.execute("""SELECT distinct bundle_id from dimm""")
        all_rows = cursor.fetchall()
        for row in all_rows:
            # logfunc(row[0])
            distinctbundle = row[0]
            if row[0] == "":
                continue
            else:
                f3 = open(
                    os.path.join(
                        report_folder, "Apps_Historical", distinctbundle + ".txt"
                    ),
                    "w+",
                    encoding="utf8",
                )  # Create historical app report per app
                cursor.execute(
                    """SELECT * from dimm where bundle_id=? order by time_stamp DESC""",
                    (distinctbundle,),
                )  # Query to create app history per bundle_id
                all_rows_hist = cursor.fetchall()
                for row in all_rows_hist:
                    # logfunc(row[0], row[1], row[2], row[3])
                    tofile3 = row[0] + " " + row[1] + " " + row[2] + " " + row[3] + "\n"
                    f3.write(tofile3)
            f3.close()
            historicalcount = historicalcount + 1

        list = []
        data_list = []
        path = os.path.join(report_folder, "Apps_Historical")
        files = os.listdir(path)
        for name in files:
            bun = f'{name}'
            appendval = f'<a href = "./Mobile Installation Logs/Apps_Historical/{name}" style = "color:blue" target="content">Report</a>'
            data_list.append((bun, appendval))

        location = f'{filename}'
        description = 'Historical App report from the Mobile Installation Logs. All timestamps are in Local Time'
        report = ArtifactHtmlReport('Apps - Historical')
        report.start_artifact_report(report_folder, 'Apps - Historical', description)
        report.add_script()
        data_headers = ('Bundle ID', 'Report Link')
        tsv_data_headers = ('Bundle ID', 'Report Link')
        report.write_artifact_data_table(
            data_headers, data_list, location, html_escape=False
        )
        report.end_artifact_report()

        tsvname = 'Mobile Installation Logs - History'
        tsv(report_folder, tsv_data_headers, tsv_tml_data_list, tsvname)
        tlactivity = 'Mobile Installation Logs - History'
        tml_data_headers = ('Timestamp', 'Event', 'Bundle ID', 'Event Path')
        timeline(report_folder, tlactivity, tsv_tml_data_list, tml_data_headers)

        # All event historical in html report
        description = 'Historical App report from the Mobile Installation Logs. All timestamps are in Local Time'
        report = ArtifactHtmlReport('Apps - Historical')
        report.start_artifact_report(
            report_folder, 'Apps - Historical Combined', description
        )
        report.add_script()
        data_headers = ('Timestamp', 'Event', 'Bundle ID', 'Event Path')
        report.write_artifact_data_table(data_headers, tsv_tml_data_list, location)
        report.end_artifact_report()

        # Query to create system events
        data_list_reboots = []
        cursor.execute(
            """SELECT * from dimm where action ='Reboot detected' order by time_stamp DESC"""
        )
        all_rows = cursor.fetchall()
        for row in all_rows:
            # logfunc(row[0])
            # logfunc(row[0], row[1], row[2], row[3])
            data_list_reboots.append((row[0], row[1]))
            sysstatecount = sysstatecount + 1

        if len(all_rows) > 0:
            location = f'{filename}'
            description = 'Reboots detected in Local Time.'
            report = ArtifactHtmlReport('State - Reboots')
            report.start_artifact_report(report_folder, 'State - Reboots', description)
            report.add_script()
            data_headers_reboots = ('Timestamp (Local Time)', 'Description')
            report.write_artifact_data_table(
                data_headers_reboots, data_list_reboots, location
            )
            report.end_artifact_report()

            tsvname = 'Mobile Installation Logs - Reboots'
            tsv(report_folder, data_headers_reboots, data_list_reboots, tsvname)

        logfunc(f"Total apps: {totalapps}")
        logfunc(f"Total installed apps: {installedcount}")
        logfunc(f"Total uninstalled apps: {uninstallcount}")
        logfunc(f"Total historical app reports: {historicalcount}")
        logfunc(f"Total system state events: {sysstatecount}")

        """
        data_headers_reboots = ('Timestamp (Local Time)', 'Description')
        tsv_data_headers = ('Timestamp (Local Time)', 'Action', 'Bundle ID', 'Path')

        tsvname = 'Mobile Installation Logs - Reboots'
        tsv(report_folder, data_headers_reboots, data_list_reboots, tsvname)

        tlactivity = 'Mobile Installation Logs - Reboots'
        timeline(report_folder, tlactivity, data_list_reboots, data_headers_reboots)

        tsvname = 'Mobile Installation Logs - History'
        tsv(report_folder, tsv_data_headers, tsv_tml_data_list, tsvname)

        tlactivity = 'Mobile Installation Logs - History'
        timeline(report_folder, tlactivity, tsv_tml_data_list, tsv_data_headers)
        """


"""

    x = 0
    data_list =[]
    for file_found in files_found:
        x = x + 1
        sx = str(x)
        journalName = os.path.basename(file_found)
        outputpath = os.path.join(report_folder, sx+'_'+journalName+'.txt') # name of file in txt
        # linkpath = os.path.basename(
        level2, level1 = (os.path.split(outputpath))
        level2 = (os.path.split(level2)[1])
        final = level2+'/'+level1
        with open(outputpath, 'w') as g:
            for s in strings(file_found):
                g.write(s)
                g.write('\n')

        out = (f'<a href="{final}" style = "color:blue" target="_blank">{journalName}</a>')

        data_list.append((out, file_found))

    location =''
    description = 'ASCII and Unicode strings extracted from SQLite journaing files.'
    report = ArtifactHtmlReport('Strings - SQLite Journal')
    report.start_artifact_report(report_folder, 'Strings - SQLite Journal', description)
    report.add_script()
    data_headers = ('Report', 'Location')
    report.write_artifact_data_table(data_headers, data_list, location, html_escape=False)
    report.end_artifact_report()
"""


def month_converter(month):
    months = [
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec",
    ]
    month = months.index(month) + 0
    if month < 9:
        month = f"{month:01d}"
    return month


def day_converter(day):
    day = int(day)
    if day < 9:
        day = f"{day:02d}"
    return day



from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv, timeline, open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class Notes(ab.Artifact):

    _name = 'Notes'
    _search_dirs = '*/NoteStore.sqlite'
    _category = 'Notes'

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()
            cursor.execute(
                """
                SELECT
                DATETIME(TabA.ZCREATIONDATE1+978307200,'UNIXEPOCH'),
                TabA.ZTITLE1,
                TabB.ZTITLE2,
                TabC.ZNAME,
                DATETIME(TabA.ZMODIFICATIONDATE1+978307200,'UNIXEPOCH'),
                case TabA.ZISPASSWORDPROTECTED
                when 0 then "No"
                when 1 then "Yes"
                end,
                case TabA.ZMARKEDFORDELETION
                when 0 then "No"
                when 1 then "Yes"
                end,
                case TabA.ZISPINNED
                when 0 then "No"
                when 1 then "Yes"
                end,
                TabD.ZFILESIZE,
                TabD.ZTYPEUTI,
                DATETIME(TabD.ZCREATIONDATE+978307200,'UNIXEPOCH'),
                DATETIME(TabD.ZMODIFICATIONDATE+978307200,'UNIXEPOCH')
                FROM ZICCLOUDSYNCINGOBJECT TabA
                INNER JOIN ZICCLOUDSYNCINGOBJECT TabB on TabA.ZFOLDER = TabB.Z_PK
                INNER JOIN ZICCLOUDSYNCINGOBJECT TabC on TabA.ZACCOUNT3 = TabC.Z_PK
                LEFT JOIN ZICCLOUDSYNCINGOBJECT TabD on TabA.Z_PK = TabD.ZNOTE
                WHERE TabA.ZTITLE1 <> ''
                """
            )

            all_rows = cursor.fetchall()
        if len(all_rows) > 0:
            for row in all_rows:
                if row[8] is not None:
                    filesize = '.'.join(
                        str(row[8])[i : i + 3] for i in range(0, len(str(row[8])), 3)
                    )
                else:
                    filesize = ''
                data_list.append(
                    (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        filesize,
                        row[9],
                        row[10],
                        row[11],
                    )
                )

        if len(data_list) > 0:
            report = ArtifactHtmlReport('Notes')
            report.start_artifact_report(report_folder, 'Notes')
            report.add_script()
            data_headers = (
                'Creation Date',
                'Note',
                'Folder',
                'Storage Place',
                'Last Modified',
                'Password Protected',
                'Marked for Deletion',
                'Pinned',
                'Attachment Size in KB',
                'Attachment Type',
                'Attachment Creation Date',
                'Attachment Last Modified',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Notes'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Notes'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Notes available')

        db.close()



import logging
import plistlib
from dataclasses import dataclass
import os
from xleapp import Artifact, WebIcon, Search, timed


@dataclass
class KnownNetwork:
    ssid: str = ""
    bssid: str = ""
    net_usage: str = ""
    country_code: str = ""
    device_name: str = ""
    manufacturer: str = ""
    serial_number: str = ""
    model_name: str = ""
    last_joined: str = ""
    last_updated: str = ""
    last_auto_joined: str = ""
    enabled: str = ""
    wnpmd: str = ""
    plist: str = ""

    def __init__(self, network: dict) -> None:

        self.ssid = str(network.get("SSID_STR", ""))
        self.bssid = str(network.get("BSSID", ""))
        self.net_usage = str(network.get("networkUsage", ""))
        self.country_code = str(
            network.get("80211D_IE", {}).get("IE_KEY_80211D_COUNTRY_CODE", ""),
        )
        self.last_updated = str(network.get("lastUpdated", ""))
        self.wnpmd = str(
            network.get("WiFiNetworkPasswordModificationDate", ""),
        )
        self.enabled = network.get("enabled", "")

        wps_prob_resp_ie = network.get("WPS_PROB_RESP_IE", "")
        if bool(wps_prob_resp_ie):
            self.device_name = wps_prob_resp_ie.get("IE_KEY_WPS_DEV_NAME", "")
            self.manufacturer = wps_prob_resp_ie.get("IE_KEY_WPS_MANUFACTURER", "")
            self.serial_number = wps_prob_resp_ie.get("IE_KEY_WPS_SERIAL_NUM", "")
            self.model_name = wps_prob_resp_ie.get(
                "IE_KEY_WPS_MODEL_NAME",
                "",
            )

    def attributes(self) -> list[str]:
        return [val for _, val in self.__dict__.items()]


@dataclass
class ScannedNetwork:
    ssid: str = ""
    bssid: str = ""
    added_at = ""
    last_joined: str = ""
    last_updated: str = ""
    private_mac_in_use = ""
    private_mac_value = ""
    private_mac_valid = ""
    in_known_networks = ""
    last_auto_joined: str = ""
    plist: str = ""

    def __init__(self, network: dict) -> None:
        self.ssid = str(network.get("SSID_STR", ""))
        self.bssid = str(network.get("BSSID", ""))
        self.last_updated = str(network.get("lastUpdated", ""))
        self.last_joined = str(network.get("lastJoined", ""))
        self.added_at = str(network.get("addedAt", ""))
        self.in_known_networks = str(
            network.get(
                "PresentInKnownNetworks",
                "",
            ),
        )

        private_mac_address = network.get("PRIVATE_MAC_ADDRESS", "")
        if bool(private_mac_address):
            self.private_mac_in_use = str(
                _bytes_to_mac_address(
                    private_mac_address.get("PRIVATE_MAC_ADDRESS_IN_USE", ""),
                ),
            )
            self.private_mac_value = getattr(
                str(
                    _bytes_to_mac_address(
                        private_mac_address.get("PRIVATE_MAC_ADDRESS_VALUE", ""),
                    ),
                ),
                "",
            )
            self.private_mac_valid = getattr(
                str(
                    private_mac_address.get("PRIVATE_MAC_ADDRESS_VALID", ""),
                ),
                "",
            )

    def attributes(self) -> list[str]:
        return [val for _, val in self.__dict__.items()]


def hexify_byte(byte_to_convert):
    to_return = hex(byte_to_convert).replace("0x", "")
    if len(to_return) < 2:
        to_return = "0" + to_return

    return to_return


def _bytes_to_mac_address(encoded_bytes):
    to_return = ""
    to_return = (
        hexify_byte(encoded_bytes[0]) + ":" + hexify_byte(encoded_bytes[1]) + ":"
    )
    to_return = (
        to_return
        + hexify_byte(encoded_bytes[2])
        + ":"
        + hexify_byte(encoded_bytes[3])
        + ":"
    )
    to_return = (
        to_return + hexify_byte(encoded_bytes[4]) + ":" + hexify_byte(encoded_bytes[5])
    )
    return to_return


@dataclass
class AppleWifiKnownNetworks(Artifact):
    def __post_init__(self):
        self.name = "Wifi Known Networks"
        self.description = (
            "WiFi known networks data. Dates are taken straight from the source plist."
        )
        self.category = "Locations"
        self.report_headers = (
            "SSID",
            "BSSID",
            "Network Usage",
            "Country Code",
            "Device Name",
            "Manufacturer",
            "Serial Number",
            "Model Name",
            "Last Joined",
            "Last Auto Joined",
            "Last Updated",
            "Enabled",
            "WiFi Network Password Modification Date",
            "File",
        )
        self.report_title = "WiFi Known Networks"
        self.web_icon = WebIcon.WIFI

    @timed
    @Search(
        "**/com.apple.wifi.plist",
        "**/com.apple.wifi-networks.plist.backup",
        "**/com.apple.wifi.known-networks.plist",
        "**/com.apple.wifi-private-mac-networks.plist",
    )
    def process(self):

        for fp in self.found:
            deserialized = plistlib.load(fp())

            known_networks = []
            try:
                for known_network in deserialized["List of known networks"]:
                    network = KnownNetwork(known_network)
                    network.plist = fp().name
                    known_networks.append(network.attributes())
            except KeyError:
                self.log(
                    logging.INFO, "No networks found in plist.", location="no_filter"
                )

        self.data = known_networks


@dataclass
class AppleWifiScannedPrivate(Artifact):
    def __post_init__(self):
        self.name = "Wifi Networks Scanned (private)"
        self.description = 'WiFi networks scanned while using fake ("private") MAC address. Dates are taken straight from the source plist.'
        self.category = "Locations"
        self.report_headers = (
            "SSID",
            "BSSID",
            "Added At",
            "Last Joined",
            "Last Updated",
            "MAC Used For Network",
            "Private MAC Computed For Network",
            "MAC Valid",
            "In Known Networks",
            "File",
        )
        self.report_title = "Wifi Networks Scanned (private)"
        self.web_icon = WebIcon.WIFI

    @timed
    @Search(
        "**/com.apple.wifi.plist",
        "**/com.apple.wifi-networks.plist.backup",
        "**/com.apple.wifi.known-networks.plist",
        "**/com.apple.wifi-private-mac-networks.plist",
    )
    def process(self):
        for fp in self.found:
            deserialzied = plistlib.load(fp(), fmt=plistlib.FMT_BINARY)

            scanned_networks = []
            try:
                for scanned_network in deserialzied[
                    "List of scanned networks with private mac"
                ]:
                    network = ScannedNetwork(scanned_network)
                    network.plist = fp().name
                    scanned_networks.append(network.attributes())
            except KeyError:
                self.log(logging.INFO, "-> No private networks scanned in plist file.")

        self.data = scanned_networks



from os.path import dirname

from html_report.artifact_report import ArtifactHtmlReport
from helpers import open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class Reminders(ab.Artifact):
    _name = 'Reminders'
    _search_dirs = '**/Reminders/Container_v1/Stores/*.sqlite'
    _category = 'Reminders'

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()
            cursor.execute(
                """
                SELECT
                DATETIME(ZCREATIONDATE+978307200,'UNIXEPOCH'),
                DATETIME(ZLASTMODIFIEDDATE+978307200,'UNIXEPOCH'),
                ZNOTES,
                ZTITLE1
                FROM ZREMCDOBJECT
                WHERE ZTITLE1 <> ''
                """
            )

            all_rows = cursor.fetchall()

        if len(all_rows) > 0:
            location_file_found = file_found.split('Stores/', 1)[1]
            for row in all_rows:
                data_list.append((row[0], row[3], row[2], row[1], location_file_found))

            dir_file_found = dirname(file_found).split('Stores', 1)[0] + 'Stores'

            report = ArtifactHtmlReport('Reminders')
            report.start_artifact_report(report_folder, 'Reminders')
            report.add_script()
            data_headers = (
                'Creation Date',
                'Title',
                'Note to Reminder',
                'Last Modified',
                'File Location',
            )
            report.write_artifact_data_table(data_headers, data_list, dir_file_found)
            report.end_artifact_report()

            tsvname = 'Reminders'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Reminders'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Reminders available')

        db.close()



import os
import plistlib

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv

from artifacts.Artifact import Artifact


class MobileBackup(ab.Artifact):
    _name = 'Mobile Backup'
    _search_dirs = '*/Preferences/com.apple.MobileBackup.plist'
    _category = 'Mobile Backup'

    # Backup version of iOS, iOS version installed at the time of recovery,
    # recovery date, and whether the backup was restored from iCloud.
    def get(self, files_found, seeker):
        data_list = []
        file_found = str(files_found[0])

        with open(file_found, 'rb') as fp:
            pl = plistlib.load(fp)

            if 'BackupStateInfo' in pl.keys():
                for key, val in pl['BackupStateInfo'].items():
                    if key == 'isCloud':
                        data_list.append((key, val))
                    if key == 'date':
                        data_list.append((key, val))
            else:
                pass

            if 'RestoreInfo' in pl.keys():
                for key, val in pl['RestoreInfo'].items():
                    if key == 'BackupBuildVersion':
                        data_list.append((key, val))
                    if key == 'DeviceBuildVersion':
                        data_list.append((key, val))
                    if key == 'WasCloudRestore':
                        data_list.append((key, val))
                    if key == 'RestoreDate':
                        data_list.append((key, val))

        report = ArtifactHtmlReport('Mobile Backup')
        report.start_artifact_report(report_folder, 'Mobile Backup')
        report.add_script()
        data_headers = ('Key', 'Value')
        report.write_artifact_data_table(data_headers, data_list, file_found)
        report.end_artifact_report()

        tsvname = 'Mobile Backup'
        tsv(report_folder, data_headers, data_list, tsvname)



from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv
from xleapp.html_report import Icon

from artifacts.Artifact import Artifact


class ConDev(ab.Artifact):

    _name = 'Connected Devices'
    _search_dirs = '**/iTunes_Control/iTunes/iTunesPrefs'
    _category = 'Connected to'
    _web_icon = WebIcon.ZAP

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        with open(file_found, "rb") as f:
            data = f.read()
            userComps = ""

            logfunc("Data being interpreted for FRPD is of type: " + str(type(data)))
            x = type(data)
            byteArr = bytearray(data)
            userByteArr = bytearray()

            magicOffset = byteArr.find(b"\x01\x01\x80\x00\x00")
            magic = byteArr[magicOffset : magicOffset + 5]

            flag = 0

            if magic == b"\x01\x01\x80\x00\x00":
                logfunc(
                    "Found magic bytes in iTunes Prefs FRPD... Finding Usernames and Desktop names now"
                )
                for x in range(int(magicOffset + 92), len(data)):
                    if (data[x]) == 0:
                        x = int(magicOffset) + 157
                        if userByteArr.decode() == "":
                            continue
                        else:
                            if flag == 0:
                                userComps += userByteArr.decode() + " - "
                                flag = 1
                            else:
                                userComps += userByteArr.decode() + "\n"
                                flag = 0
                            userByteArr = bytearray()
                            continue
                    else:
                        char = data[x]
                        userByteArr.append(char)

        report = ArtifactHtmlReport('Connected Devices')
        report.start_artifact_report(report_folder, 'Connected Devices')
        report.add_script()
        data_list = []
        data_list.append((userComps,))
        data_headers = ('User & Computer Names',)
        report.write_artifact_data_table(data_headers, data_list, file_found)
        report.end_artifact_report()

        tsvname = 'Connected Devices'
        tsv(report_folder, data_headers, data_list, tsvname)



from itertools import chain

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class Health(ab.Artifact):

    _name = 'Health Data'
    _search_dirs = '**/healthdb_secure.sqlite'
    _category = 'Health Data'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            datetime(samples.end_date + 978307200, 'unixepoch'),
            metadata_values.numerical_value,
            case workouts.activity_type
            when 63 then "high intensity interval training (hiit)"
            when 37 then "indoor / outdoor run"
            when 3000 then "other"
            when 52 then "indoor / outdoor walk"
            when 20 then "functional training"
            when 13 then "indoor cycle"
            when 16 then "elliptical"
            when 35 then "rower"
            else "unknown"
            end,
            workouts.duration / 60.00,
            workouts.total_energy_burned,
            workouts.total_distance,
            workouts.total_distance*0.621371,
            case workouts.goal_type
            when 2 then "minutes"
            when 0 then "open"
            end,
            workouts.goal,
            workouts.total_flights_climbed,
            workouts.total_w_steps
            from
            samples, metadata_values, metadata_keys, workouts
            where metadata_values.object_id = samples.data_id
            and metadata_keys.rowid = metadata_values.key_id
            and  workouts.data_id = samples.data_id
            and workouts.activity_type not null and key is "_HKPrivateWorkoutAverageHeartRate"
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                report = ArtifactHtmlReport('Health Workout Cadence')
                report.start_artifact_report(report_folder, 'Workout Cadence')
                report.add_script()
                data_headers = (
                    'Start Date',
                    'End Date',
                    'Strides per Min.',
                    'Workout Type',
                    'Duration in Mins.',
                    'Calories Burned',
                    'Distance in KM',
                    'Distance in Miles',
                    'Goal Type',
                    'Goal',
                    'Flights Climbed',
                    'Steps',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Workout Cadence'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Workout Cadence'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Workout Cadence')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            datetime(samples.end_date + 978307200, 'unixepoch'),
            quantity,
            quantity*3.28084,
            (samples.end_date-samples.start_date)
            from
            samples
            left join
            quantity_samples
            on samples.data_id = quantity_samples.data_id
            left join
            correlations
            on samples.data_id = correlations.object
            where
            samples.data_type = 8
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries == 0:
                logfunc('No data available in Distance')
            else:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                description = ''
                report = ArtifactHtmlReport('Health Distance')
                report.start_artifact_report(report_folder, 'Distance', description)
                report.add_script()
                data_headers = (
                    'Start Date',
                    'End Date',
                    'Distance in Meters',
                    'Distance in Feet',
                    'Time in Seconds',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Distance'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Distance'
                timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            datetime(samples.end_date + 978307200, 'unixepoch'),
            metadata_values.numerical_value,
            (samples.end_date-samples.start_date)
            from
            samples
            left join
            metadata_values
            on metadata_values.object_id = samples.data_id
            left join
            metadata_keys
            on metadata_keys.rowid = metadata_values.key_id
            left join
            workouts
            on workouts.data_id = samples.data_id
            where
            key is "_HKPrivateMetadataKeyElectrocardiogramHeartRate"
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries == 0:
                logfunc('No data available in ECG Avg. Heart Rate')
            else:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3]))

                description = ''
                report = ArtifactHtmlReport('Health ECG Avg Heart Rate')
                report.start_artifact_report(
                    report_folder, 'ECG Avg. Heart Rate', description
                )
                report.add_script()
                data_headers = (
                    'Start Date',
                    'End Date',
                    'ECG Avg. Heart Rate',
                    'Time in Seconds',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health ECG Avg Heart Rate'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health ECG Avg Heart Rate'
                timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            datetime(samples.end_date + 978307200, 'unixepoch'),
            quantity,
            (samples.end_date-samples.start_date)
            from
            samples, quantity_samples
            where samples.data_id = quantity_samples.data_id
            and samples.data_type = 12
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries == 0:
                logfunc('No data available in Flights Climbed')
            else:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3]))

            description = ''
            report = ArtifactHtmlReport('Health Flights Climbed')
            report.start_artifact_report(report_folder, 'Flights Climbed', description)
            report.add_script()
            data_headers = (
                'Start Date',
                'End Date',
                'Flights Climbed',
                'Time in Seconds',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Health Flights Climbed'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Health Flights Climbed'
            timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            original_quantity,
            unit_strings.unit_string,
            quantity
            from
            samples, quantity_samples, unit_strings
            where samples.data_type = 5
            and quantity_samples.original_unit = unit_strings.rowid
            and samples.data_id = quantity_samples.data_id
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries == 0:
                logfunc('No data available in Heart Rate')
            else:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3]))

                description = ''
                report = ArtifactHtmlReport('Health Heart Rate')
                report.start_artifact_report(report_folder, 'Heart Rate', description)
                report.add_script()
                data_headers = ('Date', 'Heart Rate', 'Units', 'Quantity')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Heart Rate'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Heart Rate'
                timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            datetime(samples.end_date + 978307200, 'unixepoch'),
            quantity,
            (samples.end_date-samples.start_date)
            from
            samples, quantity_samples
            where samples.data_type = 75
            and samples.data_id = quantity_samples.data_id
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3]))

                report = ArtifactHtmlReport('Health Stood Up')
                report.start_artifact_report(report_folder, 'Stood Up')
                report.add_script()
                data_headers = ('Start Date', 'End Date', 'Stood Up', 'Time in Seconds')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Stood Up'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Stood Up'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Stood Up')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            SELECT
            datetime(SAMPLES.START_DATE + 978307200, 'unixepoch'),
            datetime(SAMPLES.END_DATE + 978307200, 'unixepoch'),
            QUANTITY_SAMPLES.QUANTITY,
            (SAMPLES.END_DATE - SAMPLES.START_DATE),
            DATA_PROVENANCES.ORIGIN_PRODUCT_TYPE
            FROM SAMPLES, QUANTITY_SAMPLES, DATA_PROVENANCES, OBJECTS
            WHERE SAMPLES.DATA_TYPE = 7
                AND SAMPLES.DATA_ID = QUANTITY_SAMPLES.DATA_ID
                AND SAMPLES.DATA_ID = OBJECTS.DATA_ID
                AND OBJECTS.PROVENANCE = data_provenances.ROWID
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                daily_steps_nested_list = []

                c = 0
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                    date, hour = row[0].split(' ')

                    if date not in chain(*daily_steps_nested_list):
                        daily_steps_nested_list.append([date, row[2], row[3]])
                    else:
                        for entry in daily_steps_nested_list:
                            if entry[0] == date:
                                entry[1] += row[2]
                                entry[2] += row[3]

                daily_steps_list = [tuple(t) for t in daily_steps_nested_list]

                report = ArtifactHtmlReport('Health Steps')
                report.start_artifact_report(report_folder, 'Steps')
                report.add_script()
                data_headers = (
                    'Start Date',
                    'End Date',
                    'Steps',
                    'Time in Seconds',
                    'Origin',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Steps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Steps'
                timeline(report_folder, tlactivity, data_list, data_headers)

                description = 'Cumulative total of the steps gathered from all available sources for each day'
                report = ArtifactHtmlReport('Health Steps per Day')
                report.start_artifact_report(
                    report_folder, 'Steps per Day', description
                )
                report.add_script()
                data_headers = ('Date', 'Steps', 'Time in Seconds')
                report.write_artifact_data_table(
                    data_headers, daily_steps_list, file_found
                )
                report.end_artifact_report()

                tsvname = 'Health Steps per Day'
                tsv(report_folder, data_headers, daily_steps_list, tsvname)

                tlactivity = 'Health Steps per Day'
                timeline(report_folder, tlactivity, daily_steps_list, data_headers)

            else:
                logfunc('No data available in Steps')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            SELECT
            DATETIME(SAMPLES.START_DATE + 978307200, 'UNIXEPOCH'),
            QUANTITY,
            QUANTITY*2.20462
            FROM
            SAMPLES , QUANTITY_SAMPLES
            WHERE SAMPLES.DATA_TYPE = 3
            AND "DATE" IS  NOT NULL
            and SAMPLES.DATA_ID = QUANTITY_SAMPLES.DATA_ID
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

                report = ArtifactHtmlReport('Health Weight')
                report.start_artifact_report(report_folder, 'Weight')
                report.add_script()
                data_headers = ('Date', 'Weight in KG', 'Weight in LBS')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Weight'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Weight'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Weight')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(samples.start_date + 978307200, 'unixepoch'),
            datetime(samples.end_date + 978307200, 'unixepoch'),
            metadata_values.numerical_value/100.00,
            (metadata_values.numerical_value/100.00)*3.28084,
            case workouts.activity_type
            when 63 then "high intensity interval training (hiit)"
            when 37 then "indoor / outdoor run"
            when 3000 then "other"
            when 52 then "indoor / outdoor walk"
            when 20 then "functional training"
            when 13 then "indoor cycle"
            when 16 then "elliptical"
            when 35 then "rower"
            else "unknown"
            end,
            workouts.duration / 60.00,
            workouts.total_energy_burned,
            workouts.total_distance,
            workouts.total_distance*0.621371,
            case workouts.goal_type
            when 2 then "minutes"
            when 0 then "open"
            end,
            workouts.goal,
            workouts.total_flights_climbed,
            workouts.total_w_steps
            from
            samples, metadata_values, metadata_keys, workouts
            where metadata_values.object_id = samples.data_id
            and metadata_keys.rowid = metadata_values.key_id
            and  workouts.data_id = samples.data_id
            and workouts.activity_type not null and (key is null or key is  "HKIndoorWorkout")
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                report = ArtifactHtmlReport('Health Workout General')
                report.start_artifact_report(report_folder, 'Workout General')
                report.add_script()
                data_headers = (
                    'Start Date',
                    'End Date',
                    'Workout Type',
                    'Duration in Min.',
                    'Calories Burned',
                    'Distance in KM',
                    'Distance in Miles',
                    'Total Base Energy Burned',
                    'Goal Type',
                    'Goal',
                    'Flights Climbed',
                    'Steps',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Health Workout General'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Health Workout General'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Workout General')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            SELECT
            DATETIME(SAMPLES.START_DATE + 978307200, 'UNIXEPOCH'),
            DATETIME(SAMPLES.END_DATE + 978307200, 'UNIXEPOCH'),
            cast(ROUND(QUANTITY) as int)
            FROM SAMPLES, QUANTITY_SAMPLES
            WHERE SAMPLES.DATA_TYPE = 172 AND SAMPLES.DATA_ID = QUANTITY_SAMPLES.DATA_ID
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

                report = ArtifactHtmlReport('Environmental Sound Levels')
                report.start_artifact_report(
                    report_folder, 'Environmental Sound Levels'
                )
                report.add_script()
                data_headers = ('Start Date', 'End Date', 'Sound in dBA SPL')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Environmental Sound Levels'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Environmental Sound Levels'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Environmental Sound Levels')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            SELECT
            DATETIME(SAMPLES.START_DATE + 978307200, 'UNIXEPOCH'),
            DATETIME(SAMPLES.END_DATE + 978307200, 'UNIXEPOCH'),
            (SAMPLES.END_DATE - SAMPLES.START_DATE)
            FROM SAMPLES
            WHERE SAMPLES.DATA_TYPE = 63
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

                report = ArtifactHtmlReport('Sleep Analysis')
                report.start_artifact_report(report_folder, 'Sleep Analysis')
                report.add_script()
                data_headers = ('Start Date', 'End Date', 'Time in Seconds')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Sleep Analysis'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Sleep Analysis'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Sleep Analysis')



import glob
import os
import pathlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, kmlgen, open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class OoklaSpeedtestData(ab.Artifact):
    _name = 'Ooka Speedtest'
    _search_dirs = '**/speedtest.sqlite*'
    _category = 'Applications'

    def get(files_found, report_folder, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('speedtest.sqlite'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
            datetime(("ZDATE")+strftime('%s', '2001-01-01 00:00:00'), 'unixepoch') as 'Date',
            "ZEXTERNALIP" as 'External IP Address',
            "ZINTERNALIP" as 'Internal IP Address',
            "ZCARRIERNAME" as 'Carrier Name',
            "ZISP" as 'ISP',
            "ZWIFISSID" as 'Wifi SSID',
            "ZWANTYPE" as 'WAN Type',
            CASE "ZDEVICEMODEL"
                WHEN "iPad3,1"
                    THEN "iPad 3rd Gen (Wi-Fi Only)"
                WHEN "iPad3,2"
                    THEN "iPad 3rd Gen (Wi-Fi/Cellular Verizon/GPS)"
                WHEN "iPad3,3"
                    THEN "iPad 3rd Gen (Wi-Fi/Cellular AT&T/GPS)"
                WHEN "iPad3,4"
                    THEN "iPad 4th Gen (Wi-Fi Only)"
                WHEN "iPad3,5"
                    THEN "iPad 4th Gen (Wi-Fi/AT&T/GPS)"
                WHEN "iPad3,6"
                    THEN "iPad 4th Gen (Wi-Fi/Verizon & Sprint/GPS)"
                WHEN "iPad6,11"
                    THEN "iPad 9.7 5th Gen (Wi-Fi Only)"
                WHEN "iPad6,12"
                    THEN "iPad 9.7 5th Gen (Wi-Fi/Cellular)"
                WHEN "iPad7,5"
                    THEN "iPad 9.7 6th Gen (Wi-Fi Only)"
                WHEN "iPad7,6"
                    THEN "iPad 9.7 6th Gen (Wi-Fi/Cellular)"
                WHEN "iPad7,11"
                    THEN "iPad 10.2 7th Gen (Wi-Fi Only)"
                WHEN "iPad7,12"
                    THEN "iPad 10.2 7th Gen (Wi-Fi/Cellular Global)"
                WHEN "iPad11,3"
                    THEN "iPad Air 3rd Gen (Wi-Fi Only)"
                WHEN "iPad11,4"
                    THEN "iPad Air 3rd Gen (Wi-Fi+Cell)"
                WHEN "iPad2,5"
                    THEN "iPad mini Wi-Fi Only/1st Gen"
                WHEN "iPad2,6"
                    THEN "iPad mini Wi-Fi/AT&T/GPS - 1st Gen"
                WHEN "iPad2,7"
                    THEN "iPad mini Wi-Fi/VZ & Sprint/GPS - 1st Gen"
                WHEN "iPad4,4"
                    THEN "iPad mini 2 (Retina/2nd Gen Wi-Fi Only)"
                WHEN "iPad4,5"
                    THEN "iPad mini 2 (Retina/2nd Gen Wi-Fi/Cellular)"
                WHEN "iPad4,7"
                    THEN "iPad mini 3 (Wi-Fi Only)"
                WHEN "iPad4,8"
                    THEN "iPad mini 3 (Wi-Fi/Cellular)"
                WHEN "iPad5,1"
                    THEN "iPad mini 4 (Wi-Fi Only)"
                WHEN "iPad5,2"
                    THEN "iPad mini 4 (Wi-Fi/Cellular)"
                WHEN "iPad11,1"
                    THEN "iPad mini 5th Gen (Wi-Fi Only)"
                WHEN "iPad11,2"
                    THEN "iPad mini 5th Gen (Wi-Fi+Cell)"
                WHEN "iPad6,7"
                    THEN "iPad Pro 12.9 (Wi-Fi Only)"
                WHEN "iPad6,8"
                    THEN "iPad Pro 12.9 (Wi-Fi/Cellular)"
                WHEN "iPad6,3"
                    THEN "iPad Pro 9.7 (Wi-Fi Only)"
                WHEN "iPad6,4"
                    THEN "iPad Pro 9.7 (Wi-Fi/Cellular)"
                WHEN "iPad7,3"
                    THEN "iPad Pro 10.5 (Wi-Fi Only)"
                WHEN "iPad7,4"
                    THEN "iPad Pro 10.5 (Wi-Fi/Cellular)"
                WHEN "iPad7,1"
                    THEN "iPad Pro 12.9 (Wi-Fi Only - 2nd Gen)"
                WHEN "iPad7,2"
                    THEN "iPad Pro 12.9 (Wi-Fi/Cell - 2nd Gen)"
                WHEN "iPad8,9"
                    THEN "iPad Pro 11 (Wi-Fi Only - 2nd Gen)"
                WHEN "iPad8,10"
                    THEN "iPad Pro 11 (Wi-Fi/Cell - 2nd Gen)"
                WHEN "iPad8,11"
                    THEN "iPad Pro 12.9 (Wi-Fi Only - 4th Gen)"
                WHEN "iPad8,12"
                    THEN "iPad Pro 12.9 (Wi-Fi/Cell - 4th Gen)"
                WHEN "iPhone8,4"
                    THEN "iPhone SE (United States/A1662)"
                WHEN "iPhone9,1"
                    THEN "iPhone 7 (Verizon/Sprint/China/A1660)"
                WHEN "iPhone9,3"
                    THEN "iPhone 7 (AT&T/T-Mobile/A1778)"
                WHEN "iPhone9,2"
                    THEN "iPhone 7 Plus (Verizon/Sprint/China/A1661)"
                WHEN "iPhone9,4"
                    THEN "iPhone 7 Plus (AT&T/T-Mobile/A1784)"
                WHEN "iPhone10,1"
                    THEN "iPhone 8 (Verizon/Sprint/China/A1863)"
                WHEN "iPhone10,4"
                    THEN "iPhone 8 (AT&T/T-Mobile/Global/A1905)"
                WHEN "iPhone10,2"
                    THEN "iPhone 8 Plus (Verizon/Sprint/China/A1864)"
                WHEN "iPhone10,5"
                    THEN "iPhone 8 Plus (AT&T/T-Mobile/Global/A1897)"
                WHEN "iPhone10,3"
                    THEN "iPhone X (Verizon/Sprint/China/A1865)"
                WHEN "iPhone10,6"
                    THEN "iPhone X (AT&T/T-Mobile/Global/A1901)"
                WHEN "iPhone11,2"
                    THEN "iPhone Xs (A1920/A2097/A2098/A2100)"
                WHEN "iPhone11,6"
                    THEN "iPhone Xs Max (A1921/A2101/A2101/A2104)"
                WHEN "iPhone11,8"
                    THEN "iPhone XR (A1984/A2105/A2106/A2108)"
                WHEN "iPhone12,1"
                    THEN "iPhone 11 (A2111/A2221/A2223)"
                WHEN "iPhone12,3"
                    THEN "iPhone 11 Pro (A2160/A2215/A2217)"
                WHEN "iPhone12,5"
                    THEN "iPhone 11 Pro Max (A2161/A2218/A2220)"
                        ELSE "ZDEVICEMODEL"
                END 'Device Model',
            "ZLATITUDE" as 'Latitude',
            "ZLONGITUDE" as 'Longitude',
            "ZHORIZONTALACCURACY" as 'Accuracy in Meters'
            FROM ZSPEEDTESTRESULT

            ORDER BY "ZDATE" DESC
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append(
                    (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                        row[9],
                        row[10],
                    )
                )

                description = ''
                report = ArtifactHtmlReport('Applications')
                report.start_artifact_report(
                    report_folder, 'Ookla Speedtest', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'External IP Address',
                    'Internal IP Address',
                    'Carrier Name',
                    'ISP',
                    'Wifi SSID',
                    'WAN Type',
                    'Device Model',
                    'Latitude',
                    'Longitude',
                    'Accuracy in Meters',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Ookla Speedtest Data'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Ookla Speedtest Data'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'Ookla Speedtest Data'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                logfunc('No Ookla Speedtest Application data available')

            db.close()



import glob
import json
import os
import pathlib
import plistlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, kmlgen, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class RoutineDParkedHistorical(ab.Artifact):
    _name = 'RoutineD Parked Vehicle Historical'
    _search_dirs = '**/Local.sqlite'
    _category = 'Locations'

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) < version.parse("11"):
            logfunc("Unsupported version for RoutineD Parked Historical " + iOSversion)
            return ()

        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select
        datetime(zrtvehicleeventhistorymo.zdate + 978307200, 'unixepoch'),
        datetime(zrtvehicleeventhistorymo.zlocdate + 978307200, 'unixepoch'),
        zlocuncertainty,
        zidentifier,
        zloclatitude,
        zloclongitude
        from
        zrtvehicleeventhistorymo
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

            description = ''
            report = ArtifactHtmlReport('RoutineD Parked Vehicle Historical')
            report.start_artifact_report(
                report_folder, 'Parked Vehicle Historical', description
            )
            report.add_script()
            data_headers = (
                'Timestamp',
                'Location Date',
                'Location Uncertainty',
                'Identifier',
                'Latitude',
                'Longitude',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'RoutineD Parked Vehicle Historical'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'RoutineD Parked Vehicle Historical'
            timeline(report_folder, tlactivity, data_list, data_headers)

            kmlactivity = 'RoutineD Parked Vehicle Historical'
            kmlgen(report_folder, kmlactivity, data_list, data_headers)
        else:
            logfunc('No data available in Routine Parked Vehicle Historical')

        db.close()
